% NF-kB Signaling Model version 2.1
%
% Werner, S.L., Barken, D., Hoffmann, A. 
%  Stimulus-specificity of gene expression programs determined 
%  by temporal control of IKK activity. 
% Science, 309, pp.1857-61. 2005.

% Extended by Mitchell, S for non-canonical pathway activity

% Signaling Systems Laboratory (PI: Dr. Alexander Hoffmann) 
% Department of Chemistry and Biochemistry 
% University of California at San Diego
% 
% Questions should be addressed to
% Dr. Alexander Hoffmann    ahoffmann@ucsd.edu
%
%==========================================================
% Copyright 2007   The Regents of the University of California 
% All Rights Reserved 
%
% Permission to use, copy, modify and distribute any part of this
% NF-kB Signaling Model version 2.1 software for educational, research and
% non-profit purposes, without fee, and without a written agreement is
% hereby granted, provided that the above citation and copyright notice, this 
% paragraph and the following three paragraphs appear in all copies.
% 
% Those desiring to incorporate this NF-kB Signaling Model version 2.1
% software into commercial products or use for commercial purposes should
% contact the Technology Transfer & Intellectual Property Services,
% University of California, San Diego, 9500 Gilman Drive, Mail Code 0910,
% La Jolla, CA 92093-0910, Ph: (858) 534-5815, FAX: (858) 534-7345,
% E-MAIL:invent@ucsd.edu.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
% INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS NF-KB SIGNALING
% MODEL VERSION 2.1 SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
% ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% THE NF-KB SIGNALING MODEL VERSION 2.1 SOFTWARE PROVIDED HEREIN IS ON AN
% "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
% PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
% THE UNIVERSITY OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO
% WARRANTIES OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
% PARTICULAR PURPOSE, OR THAT THE USE OF THE NF-KB SIGNALING MODEL VERSION
% 2.1 SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR OTHER RIGHTS.
%==========================================================

% SIM Function

function steadyState = nfkb_sim_21pr(params, initvalues)

    % Global Parameters
    global SIM_TIME;
    global START_TIME;
    global GP;                  GP = params;       
    global HISTORY_TIME;	    HISTORY_TIME  = [];
    global HISTORY_NFKBN;       HISTORY_NFKBN = [];
    global PHASE;
    global OUTPUT1;
    global OUTPUT2;
    global PLOT_FLAG;
    
    % Run Phase 1 (equilibrium)
    disp('Phase 1');
    PHASE       = 1;
    static      = false;    % Set to 'true' after reaching equilibrium
    count       = 1;        % Iteration Counter 
    threshold   = .0000001;        % Max value differce used by evaluate_phase1()
    options = odeset('RelTol',1e-5);
    while ~static % Iterate through Phase 1 until equilibrium is reached
        [t1, r1]    = ode15s('nfkb_ode_21pr', [START_TIME 0], initvalues,options);

        if count == 1
            OUTPUT1 = r1;
        else
            OUTPUT1 = vertcat(OUTPUT1,r1);
        end
        [static,index]=evaluate_phase1(r1, threshold);
        % Evaluate OUTPUT1 and return true if at equilibrium
        if (static) 
            START_TIME  = (count * START_TIME) - (count-1);
            static      = true;
            disp(['Equilibrium met at ' num2str(START_TIME) ' min']);

        elseif count > 100
            plot(OUTPUT1(:,index))
            error('=====> maximum phase 1 steps reached (100 steps)');
        else
            count       = count + 1;
            fprintf('Not at equilibrium count: %i. ', count)
            initvalues  = OUTPUT1(end,:);
        end
    end  
    steadyState = OUTPUT1(end,:);
    % Run Phase 2 (stimulation
    %we don't need stim phase here
%     if SIM_TIME > 0
%         disp('Phase 2');
%         PHASE       = PHASE*0 + 2;
%         initvalues  = OUTPUT1(end,:); 
%     
%         [t2, r2] = ode15s('nfkb_ode_21pr', [0 SIM_TIME], initvalues);
%     
%         % Interpolate Phase 2 Results
%         OUTPUT2 = interp1(t2,r2(:,:),0:SIM_TIME);
% 
%         % Plot Results
%         if PLOT_FLAG == 1
%             plot_results();
%         end
%     end
end

%==========================================================
% Plot Results

function plot_results()

    global SIM_TIME OUTPUT2 CURRENT_IKK;

 % Convert uM results to nM and set the time range to plot
     OUTPUT = OUTPUT2 * 1000;   % convert uM to nM concentrations
     time   = (0:SIM_TIME)/60;  % convert minutes to hours
     basal  = 1;                % index of resting state
     time_increment = 2;        % hr increment for x labels
     
 % Analyze Results
 
    %NF-kBn
    nfkbn_total = OUTPUT(:,24);

    %Total IKB protein
    ikba_total= sum(OUTPUT(:,(1:6)),2);
    ikbb_total= sum(OUTPUT(:,(8:13)),2);
    ikbe_total= sum(OUTPUT(:,(15:20)),2);
    total_ikb = ikba_total(basal) + ikbb_total(basal) + ikbe_total(basal); 
    
    % Display text results
    disp('Basal levels for the Stimulation Phase');
    disp(['ikba_total= ' num2str(ikba_total(basal))]);
    disp(['ikbb_total= ' num2str(ikbb_total(basal))]);
    disp(['ikbe_total= ' num2str(ikbe_total(basal))]);
    disp(['Total IKB= ' num2str(total_ikb)]);
    disp(['nfkbn= ' num2str(nfkbn_total(basal))]);
    
    
end

% =====================================================================     
% Evaluate Phase 1

function [static,I] = evaluate_phase1(results,threshold)
%replaced with own phase evaulation code

    changes = abs((results(end-10,:) - results(end,:)));
    
    [diff,I]=max(changes);
    fprintf('diff: %f, ', diff)
    fprintf('index: %i\n', I)
    
    if any((diff < threshold) || (diff < 1e-20))
            static = true;
    else
            static= false;
    end
        
end   